@@ -1,6 +1,4 @@
-from __future__ import absolute_import
-from __future__ import division
-# Copyright (c) 2010-2019 openpyxl
+# Copyright (c) 2010-2020 openpyxl
 
 from io import BytesIO
 
@@ -9,21 +7,6 @@ try:
 except ImportError:
     PILImage = False
 
-def bounding_box(bw, bh, w, h):
-    """
-    Returns a tuple (new_width, new_height) which has the property
-    that it fits within box_width and box_height and has (close to)
-    the same aspect ratio as the original size
-    """
-    new_width, new_height = w, h
-    if bw and new_width > bw:
-        new_width = bw
-        new_height = new_width / (w / h)
-    if bh and new_height > bh:
-        new_height = bh
-        new_width = new_height * (w / h)
-    return (new_width, new_height)
-
 
 def _import_image(img):
     if not PILImage:
@@ -45,32 +28,32 @@ class Image(object):
     def __init__(self, img):
 
         self.ref = img
-
-        # don't keep the image open
+        mark_to_close = isinstance(img, str)
         image = _import_image(img)
-        self.width = image.size[0]
-        self.height = image.size[1]
+        self.width, self.height = image.size
+
         try:
             self.format = image.format.lower()
         except AttributeError:
             self.format = "png"
-        image.close()
+        if mark_to_close:
+            # PIL instances created for metadata should be closed.
+            image.close()
 
 
     def _data(self):
         """
-        Open image and write it to a buffer when saving the workbook
+        Return image data, convert to supported types if necessary
         """
         img = _import_image(self.ref)
-        fp = None
         # don't convert these file formats
         if self.format in ['gif', 'jpeg', 'png']:
-            if img.fp:
-                img.fp.seek(0)
-                fp = img.fp
-        if not fp:
+            img.fp.seek(0)
+            fp = img.fp
+        else:
             fp = BytesIO()
-            img.save(fp, format=self.format)
+            img.save(fp, format="png")
+            fp.seek(0)
 
         return fp.read()
 
