@@ -1,14 +1,10 @@
-from __future__ import absolute_import
-# Copyright (c) 2010-2017 openpyxl
+# Copyright (c) 2010-2020 openpyxl
 
 from openpyxl.descriptors.serialisable import Serialisable
 from openpyxl.descriptors import (
     Descriptor,
     Alias,
     Typed,
-    Set,
-    Float,
-    DateTime,
     Bool,
     Integer,
     NoneSet,
@@ -46,7 +42,7 @@ class TableStyleInfo(Serialisable):
 
     tagname = "tableStyleInfo"
 
-    name = NoneSet(values=TABLESTYLES + PIVOTSTYLES)
+    name = String(allow_none=True)
     showFirstColumn = Bool(allow_none=True)
     showLastColumn = Bool(allow_none=True)
     showRowStripes = Bool(allow_none=True)
@@ -207,7 +203,7 @@ class Table(Serialisable):
     tagname = "table"
 
     id = Integer()
-    name = TableNameDescriptor(allow_none=True)
+    name = String(allow_none=True)
     displayName = TableNameDescriptor()
     comment = String(allow_none=True)
     ref = CellRange()
@@ -362,4 +358,23 @@ class TablePartList(Serialisable):
     def __bool__(self):
         return bool(self.tablePart)
 
-    __nonzero__ = __bool__
+
+class TableList(dict):
+
+
+    def add(self, table):
+        if not isinstance(table, Table):
+            raise TypeError("You can only add tables")
+        self[table.name] = table
+
+
+    def get(self, name=None, table_range=None):
+        if name is not None:
+            return super().get(name)
+        for table in self.values():
+            if table_range == table.ref:
+                return table
+
+
+    def items(self):
+        return [(name, table.ref) for name, table in super().items()]
